# Deprecated and to be removed in exspy 1.0

import warnings

from hyperspy.exceptions import VisibleDeprecationWarning

from exspy.material import (
    atomic_to_weight,
    density_of_mixture,
    mass_absorption_coefficient,
    mass_absorption_mixture,
    weight_to_atomic,
)


__all__ = [
    "atomic_to_weight",
    "density_of_mixture",
    "mass_absorption_coefficient",
    "mass_absorption_mixture",
    "weight_to_atomic",
]


def __dir__():
    return sorted(__all__)


warnings.warn(
    "This module is deprecated, use `exspy.material` instead. "
    "It will be removed in exspy 1.0.",
    VisibleDeprecationWarning,
)



# -*- coding: utf-8 -*-
# Copyright 2007-2024 The eXSpy developers
#
# This file is part of eXSpy.
#
# eXSpy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# eXSpy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with eXSpy. If not, see <https://www.gnu.org/licenses/#GPL>.


from exspy._misc.eels.effective_angle import effective_angle
from exspy._misc.eels.electron_inelastic_mean_free_path import (
    iMFP_angular_correction,
    iMFP_Iakoubovskii,
    iMFP_TPP2M,
)
from exspy._misc.eels.tools import (
    get_edges_near_energy,
    get_info_from_edges,
)


__all__ = [
    "effective_angle",
    "get_edges_near_energy",
    "get_info_from_edges",
    "iMFP_angular_correction",
    "iMFP_Iakoubovskii",
    "iMFP_TPP2M",
]


def __dir__():
    return sorted(__all__)



import warnings

from hyperspy.exceptions import VisibleDeprecationWarning

from exspy._misc.eds.utils import (
    cross_section_to_zeta,
    electron_range,
    get_xray_lines_near_energy,
    take_off_angle,
    xray_range,
    zeta_to_cross_section,
)


warnings.warn(
    "This module is deprecated, use `exspy.utils.eds` instead. "
    "It will be removed in exspy 1.0.",
    VisibleDeprecationWarning,
)

__all__ = [
    "cross_section_to_zeta",
    "electron_range",
    "get_xray_lines_near_energy",
    "take_off_angle",
    "xray_range",
    "zeta_to_cross_section",
]


def __dir__():
    return sorted(__all__)



# -*- coding: utf-8 -*-
# Copyright 2007-2024 The eXSpy developers
#
# This file is part of eXSpy.
#
# eXSpy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# eXSpy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with eXSpy. If not, see <https://www.gnu.org/licenses/#GPL>.

import importlib
import logging

_logger = logging.getLogger(__name__)


def parse_component_module(module):
    """Check if numexpr is installed, if not fall back to numpy"""
    if module == "numexpr":
        numexpr_spec = importlib.util.find_spec("numexpr")
        if numexpr_spec is None:
            module = "numpy"
            _logger.warning(
                "Numexpr is not installed, falling back to numpy, "
                "which is slower to calculate model."
            )

    return module



# Deprecated and to be removed in exspy 1.0

import warnings

from hyperspy.exceptions import VisibleDeprecationWarning

from exspy.utils.eels import (
    effective_angle,
    get_edges_near_energy,
    get_info_from_edges,
)


__all__ = [
    "effective_angle",
    "get_edges_near_energy",
    "get_info_from_edges",
]


def __dir__():
    return sorted(__all__)


warnings.warn(
    "This module is deprecated, use `exspy.utils.eels` instead. "
    "It will be removed in exspy 1.0.",
    VisibleDeprecationWarning,
)



# -*- coding: utf-8 -*-
# Copyright 2007-2024 The eXSpy developers
#
# This file is part of eXSpy.
#
# eXSpy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# eXSpy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with eXSpy. If not, see <https://www.gnu.org/licenses/#GPL>.

from pathlib import Path

import hyperspy.api as hs
import numpy as np


TEST_DATA_DIR = Path(__file__).parent


def test_restore_EELS_model(tmp_path):
    s = hs.load(TEST_DATA_DIR / "coreloss_spectrum.msa", signal_type="EELS")
    ll = hs.load(TEST_DATA_DIR / "lowloss_spectrum.msa", signal_type="EELS")

    s.add_elements(("Mn", "O"))
    s.set_microscope_parameters(
        beam_energy=300, convergence_angle=24.6, collection_angle=13.6
    )

    m = s.create_model(low_loss=ll)
    m.enable_fine_structure()
    m.multifit(kind="smart")

    model_name = "fit1"
    m.store(model_name)
    fname = tmp_path / "test_save_eelsmodel.hspy"
    s.save(tmp_path / fname)
    m2 = s.models.restore(model_name)

    np.testing.assert_allclose(m.as_signal(), m2.as_signal())

    s2 = hs.load(fname)
    m3 = s2.models.restore(model_name)
    np.testing.assert_allclose(m.as_signal(), m3.as_signal())


def test_restore_EELS_model_dirac(tmp_path):
    s = hs.load(TEST_DATA_DIR / "coreloss_spectrum.msa", signal_type="EELS")
    ll = hs.load(TEST_DATA_DIR / "lowloss_spectrum.msa", signal_type="EELS")

    s.add_elements(("Mn", "O"))
    s.set_microscope_parameters(
        beam_energy=300, convergence_angle=24.6, collection_angle=13.6
    )

    m = s.create_model(low_loss=ll, GOS="dirac")
    m.enable_fine_structure()
    m.multifit(kind="smart")

    model_name = "fit1"
    m.store(model_name)
    fname = tmp_path / "test_save_eelsmodel.hspy"
    s.save(tmp_path / fname)
    m2 = s.models.restore(model_name)

    np.testing.assert_allclose(m.as_signal(), m2.as_signal())

    s2 = hs.load(fname)
    m3 = s2.models.restore(model_name)
    np.testing.assert_allclose(m.as_signal(), m3.as_signal())
